###########################################################################
#
# screenwidget.py
#
# A widget to display colour components from an image using independently
# selected colors. Controls are provided to allow the image to be inverted, and
# the color to be selection via a standard dialog. The image is displayed in a
# label widget.
#
# Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
#
# PyQt port in September 2010 by Hans-Peter Jansen <hpj@urpla.net>
#
###########################################################################

from PyQt4 import QtCore, QtGui

def tr(msg):
    return QtCore.QCoreApplication.translate("ScreenWidget", msg)


class ScreenWidget(QtGui.QFrame):
    # separation
    Cyan, Magenta, Yellow = range(3)
    convertMap = {
        Cyan: QtGui.qRed,
        Magenta: QtGui.qGreen,
        Yellow: QtGui.qBlue,
    }

    imageChanged = QtCore.pyqtSignal()

    def __init__(self, parent, initialColor, name, mask, labelSize):
        """ Initializes the paint color, the mask color (cyan, magenta,
            or yellow), connects the color selector and invert checkbox to functions,
            and creates a two-by-two grid layout.
        """
        self.originalImage = QtGui.QImage()
        self.newImage = QtGui.QImage()
        super(ScreenWidget, self).__init__(parent)
        self.paintColor = initialColor
        self.maskColor = mask
        self.inverted = False
        self.imageLabel = QtGui.QLabel()
        self.imageLabel.setFrameShadow(QtGui.QFrame.Sunken)
        self.imageLabel.setFrameShape(QtGui.QFrame.StyledPanel)
        self.imageLabel.setMinimumSize(labelSize)
        self.nameLabel = QtGui.QLabel(name)
        self.colorButton = QtGui.QPushButton(tr("Modify..."))
        self.colorButton.setBackgroundRole(QtGui.QPalette.Button)
        self.colorButton.setMinimumSize(32, 32)
        palette = QtGui.QPalette(self.colorButton.palette())
        palette.setColor(QtGui.QPalette.Button, initialColor)
        self.colorButton.setPalette(palette)
        self.invertButton = QtGui.QPushButton(tr("Invert"))
        #self.invertButton.setToggleButton(True)
        #self.invertButton.setOn(self.inverted)
        self.invertButton.setEnabled(False)
        self.colorButton.clicked.connect(self.setColor)
        self.invertButton.clicked.connect(self.invertImage)
        gridLayout = QtGui.QGridLayout()
        gridLayout.addWidget(self.imageLabel, 0, 0, 1, 2)
        gridLayout.addWidget(self.nameLabel, 1, 0)
        gridLayout.addWidget(self.colorButton, 1, 1)
        gridLayout.addWidget(self.invertButton, 2, 1, 1, 1)
        self.setLayout(gridLayout)


    def createImage(self):
        """ Creates a new image by separating out the cyan, magenta, or yellow
            component, depending on the mask color specified in the constructor.
            The amount of the component found in each pixel of the image is used
            to determine how much of a user-selected ink is used for each pixel
            in the new image for the label widget.
        """
        self.newImage = newImage = self.originalImage.copy()
        # Create CMY components for the ink being used.
        cyanInk = float(255 - QtGui.QColor(self.paintColor).red()) / 255.0
        magentaInk = float(255 - QtGui.QColor(self.paintColor).green()) / 255.0
        yellowInk = float(255 - QtGui.QColor(self.paintColor).blue()) / 255.0
        convert = self.convertMap[self.maskColor]
        for y in xrange(newImage.height()):
            for x in xrange(newImage.width()):
                p = self.originalImage.pixel(x, y)
                # Separate the source pixel into its cyan component.
                if self.inverted:
                    amount = convert(p)
                else:
                    amount = 255 - convert(p)
                newColor = QtGui.QColor(
                    255 - min(int(amount * cyanInk), 255),
                    255 - min(int(amount * magentaInk), 255),
                    255 - min(int(amount * yellowInk), 255))
                newImage.setPixel(x, y, newColor.rgb())

        self.imageLabel.setPixmap(QtGui.QPixmap.fromImage(newImage))


    def image(self):
        """ Returns a pointer to the modified image. """
        return self.newImage


    def invertImage(self):
        """ Sets whether the amount of ink applied to the canvas is to be inverted
            (subtracted from the maximum value) before the ink is applied.
        """
        #inverted = invertButton.isOn()
        self.inverted = not self.inverted
        self.createImage()
        self.imageChanged.emit()


    def setColor(self):
        """ Separate the current image into cyan, magenta, and yellow components.
            Create a representation of how each component might appear when applied
            to a blank white piece of paper.
        """
        newColor = QtGui.QColorDialog.getColor(self.paintColor)
        if newColor.isValid():
            self.paintColor = newColor
            palette = QtGui.QPalette(self.colorButton.palette())
            palette.setColor(QtGui.QPalette.Button, self.paintColor)
            self.colorButton.setPalette(palette)
            self.createImage()
            self.imageChanged.emit()


    def setImage(self, image):
        """ Records the original image selected by the user, creates a color
            separation, and enables the invert image checkbox.
        """
        self.originalImage = image
        self.createImage()
        self.invertButton.setEnabled(True)

