#!/usr/bin/env python
###########################################################################
#
# viewer.py
#
# Provides a main window for displaying a user-specified original image
# with three color separations in a grid layout.
# A main menu provides entries for selecting files, and adjusting the
# brightness of the separations.
#
# Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
#
# PyQt port in September 2010 by Hans-Peter Jansen <hpj@urpla.net>
#
###########################################################################

from PyQt4 import QtCore, QtGui
from screenwidget import ScreenWidget
from finalwidget import FinalWidget

def tr(msg):
    return QtCore.QCoreApplication.translate("Viewer", msg)


class Viewer(QtGui.QMainWindow):
    # brightness
    Gloom, Quarter, Half, ThreeQuarters, Full = range(5)
    # brightness value map
    brightnessValueMap = {
        Gloom: 0,
        Quarter: 64,
        Half: 128,
        ThreeQuarters: 191,
        Full: 255,
    }

    def __init__(self, path = None):
        """ Constructor initializes a default value for the brightness, creates
            the main menu entries, and constructs a central widget that contains
            enough space for images to be displayed.
        """
        self.brightness = None
        self.scaledImage = QtGui.QImage()
        self.menuMap = {}
        self.path = path
        super(Viewer, self).__init__()
        self.setWindowTitle(tr("QImage Color Separations"))
        self.brightness = 255
        self.createMenus()
        self.setCentralWidget(self.createCentralWidget())
        if path is not None:
            QtCore.QTimer.singleShot(0, lambda: self.openImageFile(path))


    def createMenus(self):
        """ Creates a main menu with two entries: a File menu, to allow the image
            to be selected, and a Brightness menu to allow the brightness of the
            separations to be changed.
            Initially, the Brightness menu items are disabled, but the first entry in
            the menu is checked to reflect the default brightness.
        """
        self.fileMenu = QtGui.QMenu(tr("&File"), self)
        self.brightnessMenu = QtGui.QMenu(tr("&Brightness"), self)
        self.openAction = self.fileMenu.addAction(tr("&Open..."))
        self.openAction.setShortcut(QtGui.QKeySequence("Ctrl+O"))
        self.saveAction = self.fileMenu.addAction(tr("&Save..."))
        self.saveAction.setShortcut(QtGui.QKeySequence("Ctrl+S"))
        self.saveAction.setEnabled(False)
        self.quitAction = self.fileMenu.addAction(tr("E&xit"))
        self.quitAction.setShortcut(QtGui.QKeySequence("Ctrl+Q"))
        self.noBrightness = self.brightnessMenu.addAction(tr("&0%"))
        self.noBrightness.setCheckable(True)
        self.quarterBrightness = self.brightnessMenu.addAction(tr("&25%"))
        self.quarterBrightness.setCheckable(True)
        self.halfBrightness = self.brightnessMenu.addAction(tr("&50%"))
        self.halfBrightness.setCheckable(True)
        self.threeQuartersBrightness = self.brightnessMenu.addAction(tr("&75%"))
        self.threeQuartersBrightness.setCheckable(True)
        self.fullBrightness = self.brightnessMenu.addAction(tr("&100%"))
        self.fullBrightness.setCheckable(True)
        self.menuMap[self.noBrightness] = self.Gloom
        self.menuMap[self.quarterBrightness] = self.Quarter
        self.menuMap[self.halfBrightness] = self.Half
        self.menuMap[self.threeQuartersBrightness] = self.ThreeQuarters
        self.menuMap[self.fullBrightness] = self.Full
        self.currentBrightness = self.fullBrightness
        self.currentBrightness.setChecked(True)
        self.brightnessMenu.setEnabled(False)
        self.menuBar().addMenu(self.fileMenu)
        self.menuBar().addMenu(self.brightnessMenu)
        self.openAction.triggered.connect(self.chooseFile)
        self.saveAction.triggered.connect(self.saveImage)
        self.quitAction.triggered.connect(QtGui.qApp.quit)
        self.brightnessMenu.triggered[QtGui.QAction].connect(self.setBrightness)


    def createCentralWidget(self):
        """ Constructs a central widget for the window consisting of a two-by-two
            grid of labels, each of which will contain an image. We restrict the
            size of the labels to 256 pixels, and ensure that the window cannot
            be resized.
        """
        frame = QtGui.QFrame(self)
        grid = QtGui.QGridLayout(frame)
        grid.setSpacing(8)
        grid.setMargin(4)
        self.layout().setSizeConstraint(QtGui.QLayout.SetFixedSize)
        labelSize = QtCore.QSize(256, 256)
        self.finalWidget = FinalWidget(frame, tr("Final image"), labelSize)
        self.cyanWidget = ScreenWidget(frame, QtCore.Qt.cyan, tr("Cyan"),
                                       ScreenWidget.Cyan, labelSize)
        self.magentaWidget = ScreenWidget(frame, QtCore.Qt.magenta, tr("Magenta"),
                                         ScreenWidget.Magenta, labelSize)
        self.yellowWidget = ScreenWidget(frame, QtCore.Qt.yellow, tr("Yellow"),
                                        ScreenWidget.Yellow, labelSize)
        self.cyanWidget.imageChanged.connect(self.createImage)
        self.magentaWidget.imageChanged.connect(self.createImage)
        self.yellowWidget.imageChanged.connect(self.createImage)
        grid.addWidget(self.finalWidget, 0, 0, QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter)
        grid.addWidget(self.cyanWidget, 0, 1, QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter)
        grid.addWidget(self.magentaWidget, 1, 0, QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter)
        grid.addWidget(self.yellowWidget, 1, 1, QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter)
        return frame


    def chooseFile(self):
        """ Provides a dialog window to allow the user to specify an image file.
            If a file is selected, the appropriate function is called to process
            and display it.
        """
        imageFile = QtGui.QFileDialog.getOpenFileName(self,
            tr("Choose an image file to open"), self.path, tr("Images (*.*)"))
        if not imageFile.isEmpty():
            self.openImageFile(imageFile)
            path = imageFile


    def setBrightness(self, action):
        """ Changes the value of the brightness according to the entry selected in the
            Brightness menu. The selected entry is checked, and the previously selected
            entry is unchecked.
            The color separations are updated to use the new value for the brightness.
        """
        if action not in self.menuMap or self.scaledImage.isNull():
            return
        self.brightness = self.brightnessValueMap.get(self.menuMap[action])
        if self.brightness is None:
            return

        self.currentBrightness.setChecked(False)
        self.currentBrightness = action
        self.currentBrightness.setChecked(True)
        self.createImage()


    def openImageFile(self, imageFile):
        """ Load the image from the file given, and create four pixmaps based
            on the original image.
            The window caption is set, and the Brightness menu enabled if the image file
            can be loaded.
        """
        originalImage = QtGui.QImage()
        if originalImage.load(imageFile):
            self.setWindowTitle(imageFile)
            #menuBar().setItemEnabled(brightnessMenuId, True)
            self.saveAction.setEnabled(True)
            self.brightnessMenu.setEnabled(True)
            self.scaledImage = originalImage.scaled(QtCore.QSize(256, 256), QtCore.Qt.KeepAspectRatio)
            self.cyanWidget.setImage(self.scaledImage)
            self.magentaWidget.setImage(self.scaledImage)
            self.yellowWidget.setImage(self.scaledImage)
            self.createImage()
        else:
            QtGui.QMessageBox.warning(self, tr("Cannot open file"),
                tr("The selected file could not be opened."),
                QtGui.QMessageBox.Cancel, QtGui.QMessageBox.NoButton, QtGui.QMessageBox.NoButton)


    def createImage(self):
        """ Creates an image by combining the contents of the three screens
            to present a page preview.
            The image associated with each screen is separated into cyan,
            magenta, and yellow components. We add up the values for each
            component from the three screen images, and subtract the totals
            from the maximum value for each corresponding primary color.
        """
        newImage = self.scaledImage.copy()
        image1 = self.cyanWidget.image()
        image2 = self.magentaWidget.image()
        image3 = self.yellowWidget.image()
        darkness = 255 - self.brightness
        for y in xrange(newImage.height()):
            for x in xrange(newImage.width()):
                # Create three screens, using the quantities of the source
                # CMY components to determine how much of each of the
                # inks are to be put on each screen.
                p1 = image1.pixel(x, y)
                cyan1 = float(255 - QtGui.qRed(p1))
                magenta1 = float(255 - QtGui.qGreen(p1))
                yellow1 = float(255 - QtGui.qBlue(p1))
                p2 = image2.pixel(x, y)
                cyan2 = float(255 - QtGui.qRed(p2))
                magenta2 = float(255 - QtGui.qGreen(p2))
                yellow2 = float(255 - QtGui.qBlue(p2))
                p3 = image3.pixel(x, y)
                cyan3 = float(255 - QtGui.qRed(p3))
                magenta3 = float(255 - QtGui.qGreen(p3))
                yellow3 = float(255 - QtGui.qBlue(p3))
                newColor = QtGui.QColor(
                    max(255 - int(cyan1 + cyan2 + cyan3) - darkness, 0),
                    max(255 - int(magenta1 + magenta2 + magenta3) - darkness, 0),
                    max(255 - int(yellow1 + yellow2 + yellow3) - darkness, 0))
                newImage.setPixel(x, y, newColor.rgb())

        self.finalWidget.setPixmap(QtGui.QPixmap.fromImage(newImage))


    def saveImage(self):
        """ Provides a dialog window to allow the user to save the image file.
        """
        imageFile = QtGui.QFileDialog.getSaveFileName(self,
            tr("Choose a filename to save the image"), "", tr("Images (*.png)"))
        info = QtCore.QFileInfo(imageFile)
        if not info.baseName().isEmpty():
            newImageFile = QtCore.QFileInfo(info.absoluteDir(),
                info.baseName() + ".png").absoluteFilePath()
            if not self.finalWidget.pixmap().save(newImageFile, "PNG"):
                QtGui.QMessageBox.warning(self, tr("Cannot save file"),
                    tr("The file could not be saved."),
                    QtGui.QMessageBox.Cancel, QtGui.QMessageBox.NoButton,
                    QtGui.QMessageBox.NoButton)
        else:
            QtGui.QMessageBox.warning(self, tr("Cannot save file"),
                tr("Please enter a valid filename."),
                QtGui.QMessageBox.Cancel, QtGui.QMessageBox.NoButton,
                QtGui.QMessageBox.NoButton)

